#!/bin/env -S deno run --allow-read

import { CircDepErr, RecipeSet, Step } from "./tree.ts"

async function main() {
	const { path } = parse_args(Deno.args)
	const content = await Deno.readTextFile(path)
	const [targets, recipes] = RecipeSet.parse(content)
	const steps = Step.resolve(targets, recipes)
	if (steps instanceof CircDepErr) return console.error("Circular recipe detected", steps.path)
	report_steps(steps)
}

function parse_args(args: string[]) {
	const invalid_usage = () => Deno.exit(console.log("Usage:\nreciper <recipe_file>") as undefined)
	if (args.length !== 1) invalid_usage()
	const [path] = args
	if (["-h", "--help"].includes(path)) invalid_usage()
	return { path }
}

function report_steps(steps: Step) {
	console.log("Steps")
	console.log()
	console.log(steps.display())
	console.log()
	console.log("Intermediaries")
	console.log()
	console.log(steps.intermediaries().map(([name, amount]) => `${amount}\t${name}`).join("\n"))
	console.log()
	console.log("Inputs")
	console.log()
	console.log(steps.inputs().map(([name, amount]) => `${amount}\t${name}`).join("\n"))
}

if (import.meta.main) await main()
